/* 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * ----------------------------------------------------------------------------
 *
 * Before you start please ensure that both the library and the board is
 * updated to the latest version.
 *
 ******************************************************************************
 * About this sketch
 ******************************************************************************
 *
 * This sketch samples sensor data and builds up a payload which is sent over a LoRaWAN Radio
 * The sketch allows you to configure the sample frequency and puts the hardware into sleep mode during
 * the remainder of the time to conserve energy. In sleep mode the device consumes around 17 microAmps.
 * 
 * The sketch makes use of the AllthingsTalk LoRaWAN SDK which collects and aggregates the data towards
 * maker.allthingstalk.com
 * 
 * Plantsenor payload
 * Example payload: 0000000141A67AE1447BF64F42787D0043828000437F00000000000000000064
 * 
 * Soil :       startbyte 0   bytelength 4 
 * Temperature: startbyte 4   bytelength 4
 * Pressure:    startbyte 8   bytelength 4
 * Humidity:    startbyte 12  bytelength 4
 * Vis:         startbyte 16  bytelength 4
 * IR:          startbyte 20  bytelength 4
 * UV:          startbyte 24  bytelength 4
 * Bat:         startbyte 28  bytelength 4
 * 
 * 
 */

#include <RTCZero.h>
#include "Wire.h"
#include <AllThingsTalk_LoRaWAN.h>
#include "keys.h"
#include <math.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>
#include "SI114X.h"
#include <Sodaq_LSM303AGR.h>


// define serial ports
#define debugSerial SerialUSB
#define loraSerial Serial1

// Using OTAA provisioning
OTAACredentials credentials(DEVEUI, APPEUI, APPKEY);
LoRaModem modem(loraSerial, debugSerial, credentials);
BinaryPayload payload;
bool toggle = false;

// Define sensor port numbers

const int soilSensor = A6;     // Grove - Soil Sensor connect to A6
#define enablePin 11


// Define Wakke Up Interval
#define WAKEUP_EVERY_SEC 00                  // Seconds part of the clock that wakes up the device    
#define WAKEUP_EVERY_MIN 05                  // Minutes part of the clock that wakes up the device
#define WAKEUP_EVERY_HR 00                   // Hours part of the clock that wakes up the device

RTCZero rtc;
volatile bool rtc_flag = false;


unsigned long time;

Adafruit_BME280 bme; // I2C

SI114X SI1145 = SI114X(); //sunlight sensor

Sodaq_LSM303AGR accelerometer;

enum LedColor {
    NONE = 0,
    RED,
    GREEN,
    BLUE,
    YELLOW,
    MAGENTA,
    CYAN,
    WHITE
};


// Interrupt routine

void alarmMatch()
{
rtc_flag = true;
}

// Define Battery settings
#define ADC_AREF 3.3f
#define BATVOLT_R1 2.0f
#define BATVOLT_R2 2.0f
#define BATVOLT_PIN BAT_VOLT

uint16_t getBatteryVoltage()
{
    uint16_t voltage = (uint16_t)((ADC_AREF / 1.023) * (BATVOLT_R1 + BATVOLT_R2) / BATVOLT_R2 * (float)analogRead(BATVOLT_PIN));
    int val = voltage*100/4095;
    if (val > 100) val = 100;
    return (val);
}



/**
 * Turns the led on according to the given color. Makes no assumptions about the status of the pins
 * i.e. it sets them every time,
 */
void setLedColor(LedColor color)
{
    pinMode(LED_RED, OUTPUT);
    pinMode(LED_GREEN, OUTPUT);
    pinMode(LED_BLUE, OUTPUT);

    digitalWrite(LED_RED, HIGH);
    digitalWrite(LED_GREEN, HIGH);
    digitalWrite(LED_BLUE, HIGH);

    switch (color)
    {
    case NONE:
        break;
    case RED:
        digitalWrite(LED_RED, LOW);
        break;
    case GREEN:
        digitalWrite(LED_GREEN, LOW);
        break;
    case BLUE:
        digitalWrite(LED_BLUE, LOW);
        break;
    case YELLOW:
        digitalWrite(LED_GREEN, LOW);
        digitalWrite(LED_RED, LOW);
        break;
    case MAGENTA:
        digitalWrite(LED_BLUE, LOW);
        digitalWrite(LED_RED, LOW);
        break;
    case CYAN:
        digitalWrite(LED_GREEN, LOW);
        digitalWrite(LED_BLUE, LOW);
        break;
    case WHITE:
        digitalWrite(LED_GREEN, LOW);
        digitalWrite(LED_RED, LOW);
        digitalWrite(LED_BLUE, LOW);
        break;
    default:
        break;
    }
}

void setup() {
  debugSerial.begin(57600);
  while ((!debugSerial)) {}

  // init the baud rate of the serial connection so that it's ok for the modem
  //wait until serial bus is available
    if (!modem.init())
  {
    exit(0);
  }
  else
  {
    modem.setPort(1);
    modem.setSpreadingFactor(9);
    modem.setAck(false); 
  }

  debugSerial.println(F("BME280 test"));

  if (! bme.begin()) {
     debugSerial.println("Could not find a valid BME280 sensor, check wiring!");
     while (1);
     }

  debugSerial.println("-- Default Test --");
  debugSerial.println("normal mode, 16x oversampling for all, filter off,");
  debugSerial.println("0.5ms standby period");
 
    
  // For more details on the following scenarious, see chapter
  // 3.5 "Recommended modes of operation" in the datasheet
    
  // weather monitoring
  debugSerial.println("-- Weather Station Scenario --");
  debugSerial.println("forced mode, 1x temperature / 1x humidity / 1x pressure oversampling,");
  debugSerial.println("filter off");
  bme.setSampling(Adafruit_BME280::MODE_FORCED,
  Adafruit_BME280::SAMPLING_X1, // temperature
  Adafruit_BME280::SAMPLING_X1, // pressure
  Adafruit_BME280::SAMPLING_X1, // humidity
  Adafruit_BME280::FILTER_OFF   );
  
  // suggested rate is 1/60Hz (1m)


  while (!SI1145.Begin()) {
    Serial.println("Si1145 is not ready!");
    delay(1000);
  }
  Serial.println("Si1145 is ready!");


  Wire.begin();// I2C for the accelerometer

  // Disable the LSM303AGR
  accelerometer.disableAccelerometer();
  accelerometer.disableMagnetometer();

  pinMode(MAG_INT, OUTPUT);
  pinMode(GPS_ENABLE, OUTPUT);

  digitalWrite(MAG_INT, LOW); // we need to make this low otherwise this pin on the LSM303AGR starts leaking current
  digitalWrite(GPS_ENABLE, LOW); // low=poweredoff, high=poweredon

  debugSerial.flush();
  debugSerial.end();
  USBDevice.detach();
  USB->DEVICE.CTRLA.reg &= ~USB_CTRLA_ENABLE; // Disable USB

  // Put the modem in sleep to save energy
  modem.sleep();
  loraSerial.flush();
  delay(1000);

  rtc.begin();
  rtc.setEpoch(0);                                           // This sets it to 2000-01-01
  rtc.setAlarmSeconds(WAKEUP_EVERY_SEC);                     // Schedule the wakeup interrupt
  rtc.setAlarmMinutes(WAKEUP_EVERY_MIN);
  rtc.setAlarmHours(WAKEUP_EVERY_HR);
  rtc.enableAlarm(rtc.MATCH_MMSS);                           // MATCH_HHMMSS
  rtc.attachInterrupt(alarmMatch);                           // Attach handler so that we can set the battery flag when the time has passed.
  
  Sensorsampling();
  // payload transmittion
  sendPayload();
  //rtc.standbyMode();

// Set the sleep mode
  SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;
// SAMD Sleep
  __WFI();
}

void Sensorsampling()

{
   digitalWrite(enablePin, HIGH);    // If you want to use the pins: 2/3,  6/7 and 8/9 you first have to set pin D11 high. 
   delay(100);
   int soil = analogRead(soilSensor);
   digitalWrite(enablePin, LOW);
   
   payload.reset();    //init payload
   payload.add(soil);

   // TPH reading
   bme.takeForcedMeasurement(); // has no effect in normal mode
   float temp = bme.readTemperature();   
   payload.add(temp);
    
   float pressure = bme.readPressure() / 100.0F;
   payload.add(pressure);

   float humidity = bme.readHumidity();
   payload.add(humidity);

   // sunlight reading

   float vis = SI1145.ReadVisible();
   float ir = SI1145.ReadIR();
   float uv = SI1145.ReadUV()/100;
   
   //the real UV value must be div 100 from the reg value , datasheet for more information.
   payload.add(vis);
   payload.add(ir);
   payload.add(uv);

   int val = getBatteryVoltage();
   payload.add(val);

}

void sendPayload()
{
     modem.wakeUp();
     setLedColor(GREEN);
     modem.send(payload);
     setLedColor(NONE);
     modem.sleep();
     loraSerial.flush();       
}


void loop()
{
  //Wake Up on time Interval
  if(rtc_flag) {
      rtc.disableAlarm();
      rtc_flag = false;
      Sensorsampling();
      sendPayload();
  
      rtc.setEpoch(0);                      // This sets it to 2000-01-01
      rtc.enableAlarm(rtc.MATCH_MMSS);      // 
      }
   rtc.standbyMode();    // Sleep until next alarm match 
   }
